@@ -1,15 +1,19 @@
-/*  $Revision: 1.5 $
+/*  $Revision: 6786 $
 **
 **  Here be values used for communicating with the server once it is
 **  running.
 */
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 /* The header for the ICC protocol is a one-byte protocol version followed
-   by a 4 byte messages length*/
+   by a 2 byte messages length*/
 #define HEADER_SIZE (sizeof (ICC_PROTOCOLTYPE) + sizeof (ICC_MSGLENTYPE))
 
-typedef short ICC_MSGLENTYPE;	/* Length code to prefix commands to
-				** the server. */
+typedef unsigned short ICC_MSGLENTYPE;	/* Length code to prefix commands to
+					** the server. */
 typedef char ICC_PROTOCOLTYPE ;
 
 /* Values for the protocol version field of the message. 8 bits wide. */
@@ -41,6 +45,7 @@ typedef char ICC_PROTOCOLTYPE ;
 #define SC_PARAM	'l'
 #define SC_PAUSE	'm'
 #define SC_PERL		'P'
+#define SC_PYTHON	'Y'
 #define SC_READERS	'v'
 #define SC_REJECT	'C'
 #define SC_RELOAD	'o'
@@ -49,10 +54,11 @@ typedef char ICC_PROTOCOLTYPE ;
 #define SC_RMGROUP	'p'
 #define SC_SEND		'A'
 #define SC_SHUTDOWN	'q'
+#define SC_STATHIST	'H'
 #define SC_STATUS	'S'
 #define SC_SIGNAL	'B'
 #define SC_THROTTLE	'r'
-#define SC_TIMER        'Z'
+#define SC_TIMER	'Z'
 #define SC_TRACE	'w'
 #define SC_XABORT	'x'
 #define SC_XEXEC	'y'
@@ -60,16 +66,27 @@ typedef char ICC_PROTOCOLTYPE ;
     /* Yes, we don't want anyone to use this. */
 #define SC_FIRSTFREE	G
 
-#define MAX_REASON_LEN 80
+#define MAX_REASON_LEN	80
 
 
-extern void	ICCsettimeout();
-extern int	ICCopen();
-extern int	ICCclose();
-extern int	ICCcommand();
-extern int	ICCcancel();
-extern int	ICCgo();
-extern int	ICCpause();
-extern int	ICCreserve();
+extern void		ICCsettimeout(int i);
+extern int		ICCopen(void);
+extern int		ICCclose(void);
+extern int		ICCcommand(char cmd, const char *argv[], char **replyp);
+extern int		ICCcancel(const char *msgid);
+extern int		ICCgo(const char *why);
+extern int		ICCpause(const char *why);
+extern int		ICCreserve(const char *why);
 
-extern char	*ICCfailure;
+extern const char	*ICCfailure;
+
+/* Use a read or recv call to read a descriptor. */
+#ifdef HAVE_UNIX_DOMAIN_SOCKETS
+# define RECVorREAD(fd, p, s)   recv((fd), (p), (s), 0)
+#else
+# define RECVorREAD(fd, p, s)   read((fd), (p), (s))
+#endif
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
