@@ -1,4 +1,4 @@
-/*  $Id: art.c 7410 2005-10-09 03:32:22Z eagle $
+/*  $Id: art.c 7748 2008-04-06 13:49:56Z iulius $
 **
 **  Article-processing.
 */
@@ -9,6 +9,7 @@
 
 #include "inn/innconf.h"
 #include "inn/wire.h"
+#include "inn/md5.h"
 #include "innd.h"
 #include "ov.h"
 #include "storage.h"
@@ -439,11 +440,17 @@ ARTstore(CHANNEL *cp)
   for (p = Article->data + cp->Start, j = 0 ; j < i ; j++) {
     switch (hp[j].index) {
       case HDR__PATH:
-	if (!data->Hassamepath || data->AddAlias) {
+	if (!data->Hassamepath || data->AddAlias || Pathcluster.used) {
 	  /* write heading data */
 	  iov[iovcnt].iov_base = (char *) p;
 	  iov[iovcnt++].iov_len = HDR(HDR__PATH) - p;
 	  arth.len += HDR(HDR__PATH) - p;
+          /* append clusterpath */
+          if (Pathcluster.used) {
+            iov[iovcnt].iov_base = Pathcluster.data;
+            iov[iovcnt++].iov_len = Pathcluster.used;
+            arth.len += Pathcluster.used;
+          }
 	  /* now append new one */
 	  iov[iovcnt].iov_base = Path.data;
 	  iov[iovcnt++].iov_len = Path.used;
@@ -455,6 +462,8 @@ ARTstore(CHANNEL *cp)
 	  }
 	  /* next to write */
 	  p = HDR(HDR__PATH);
+          if (data->Hassamecluster)
+            p += Pathcluster.used;
 	}
 	break;
       case HDR__XREF:
@@ -1552,6 +1561,88 @@ ListHas(const char **list, const char *p)
   return false;
 }
 
+/*
+**  Even though we have already calculated the Message-ID MD5sum,
+**  we have to do it again since unfortunately HashMessageID()
+**  lowercases the Message-ID first.  We also need to remain
+**  compatible with Diablo's hashfeed.
+*/
+
+static unsigned int
+HashFeedMD5(char *MessageID, unsigned int offset)
+{
+  static char LastMessageID[128];
+  static char *LastMessageIDPtr;
+  static struct md5_context context;
+  unsigned int ret;
+
+  if (offset > 12)
+    return 0;
+
+  /* Some light caching. */
+  if (MessageID != LastMessageIDPtr ||
+    strcmp(MessageID, LastMessageID) != 0) {
+    md5_init(&context);
+    md5_update(&context, (unsigned char *)MessageID, strlen(MessageID));
+    md5_final(&context);
+    LastMessageIDPtr = MessageID;
+    strncpy(LastMessageID, MessageID, sizeof(LastMessageID) - 1);
+    LastMessageID[sizeof(LastMessageID) - 1] = 0;
+  }
+
+  memcpy(&ret, &context.digest[12 - offset], 4);
+
+  return ntohl(ret);
+}
+
+/*
+** Old-style Diablo (< 5.1) quickhash.
+**
+*/
+static unsigned int
+HashFeedQH(char *MessageID, unsigned int *tmp)
+{
+  unsigned char *p;
+  int n;
+
+  if (*tmp != (unsigned int)-1)
+    return *tmp;
+
+  p = (unsigned char *)MessageID;
+  n = 0;
+  while (*p)
+    n += *p++;
+  *tmp = (unsigned int)n;
+
+  return *tmp;
+}
+
+/*
+**  Return true if an element of the HASHFEEDLIST matches
+**  the hash of the Message-ID.
+*/
+static bool
+HashFeedMatch(HASHFEEDLIST *hf, char *MessageID)
+{
+  unsigned int qh = (unsigned int)-1;
+  unsigned int h;
+
+  while (hf) {
+    if (hf->type == HASHFEED_MD5)
+      h = HashFeedMD5(MessageID, hf->offset);
+    else if (hf->type == HASHFEED_QH)
+      h = HashFeedQH(MessageID, &qh);
+    else
+      continue;
+    if ((h % hf->mod + 1) >= hf->begin &&
+        (h % hf->mod + 1) <= hf->end)
+	  return true;
+    hf = hf->next;
+  }
+
+  return false;
+}
+
 /*
 **  Propagate an article to the sites have "expressed an interest."
 */
@@ -1625,6 +1716,11 @@ ARTpropagate(ARTDATA *data, const char **hops, int hopcount, char **list,
        * cross-posting. */
       continue;
 
+    if (sp->HashFeedList &&
+      !HashFeedMatch(sp->HashFeedList, HDR(HDR__MESSAGE_ID)))
+      /* hashfeed doesn't match */
+      continue;
+
     if (list && *list != NULL && sp->Distributions &&
       !DISTwantany(sp->Distributions, list))
       /* Not in the site's desired list of distributions. */
@@ -1856,10 +1952,18 @@ ARTpost(CHANNEL *cp)
     return false;
   }
 
-  if (strncmp(Path.data, hops[0], Path.used - 1) == 0)
+  i = strlen(hops[0]);
+  if (i == Path.used - 1 &&
+    strncmp(Path.data, hops[0], Path.used - 1) == 0)
     data->Hassamepath = true;
   else
     data->Hassamepath = false;
+  if (Pathcluster.data != NULL &&
+    i == Pathcluster.used - 1 &&
+    strncmp(Pathcluster.data, hops[0], Pathcluster.used - 1) == 0)
+    data->Hassamecluster = true;
+  else
+    data->Hassamecluster = false;
   if (Pathalias.data != NULL &&
     !ListHas((const char **)hops, (const char *)innconf->pathalias))
     data->AddAlias = true;
@@ -2078,6 +2182,9 @@ ARTpost(CHANNEL *cp)
       if (data->Followcount == 0)
 	data->Followcount = data->Groupcount;
     }
+    
+    LikeNewgroup = (LikeNewgroup || strcmp(ControlWord, "checkgroups") == 0);
+    
     /* Control messages to "foo.ctl" are treated as if they were
      * posted to "foo".  I should probably apologize for all the
      * side-effects in the if. */
@@ -2099,14 +2206,15 @@ ARTpost(CHANNEL *cp)
     if ((ngp = NGfind(p)) == NULL) {
       GroupMissing = true;
       if (LikeNewgroup && Approved) {
-	/* Newgroup/rmgroup being sent to a group that doesn't exist.  Assume
-	 * it is being sent to the group being created or removed, nd send the
-	 * group to all sites that would or would have had the group if it were
-	 * created. */
-	ARTsendthegroup(*groups);
-	Accepted = true;
+        /* Checkgroups/newgroup/rmgroup being sent to a group that doesn't
+         * exist.  Assume it is being sent to the group being created or
+         * removed (or to the admin group to which the checkgroups is posted),
+         * and send it to all sites that would or would have had the group
+         * if it were created. */
+        ARTsendthegroup(*groups);
+        Accepted = true;
       } else
-	NonExist = true;
+        NonExist = true;
       ARTpoisongroup(*groups);
 
       if (innconf->mergetogroups) {
@@ -2210,8 +2318,12 @@ ARTpost(CHANNEL *cp)
     *ngptr++ = ngp;
     for (isp = ngp->Sites, i = ngp->nSites; --i >= 0; isp++) {
       if (*isp >= 0) {
-	sp = &Sites[*isp];
-	SITEmark(sp, ngp);
+        /* Checkgroups/newgroup/rmgroup posted to local.example
+         * will still be sent with the newsfeeds patterns
+         * "*,!local.*" and "*,@local.*".  So as not to propagate
+         * them, "!control,!control.*" should be added. */
+        sp = &Sites[*isp];
+        SITEmark(sp, ngp);
       }
     }
   }
