@@ -1,4 +1,4 @@
-/*  $Id: innd.h,v 1.98.2.3 2004/05/27 18:55:23 rra Exp $
+/*  $Id: innd.h 7858 2008-06-05 18:51:20Z iulius $
 **
 **  Many of the data types used here have abbreviations, such as CT
 **  for CHANNELTYPE.  Here are a list of the conventions and meanings:
@@ -177,9 +177,36 @@ typedef struct _HDRCONTENT {
 #define HDR__XCANCELEDBY	35
 #define HDR__CANCELKEY		36
 #define HDR__USER_AGENT		37
-#define HDR__X_ORIGINAL_MESSAGE_ID		38
+#define HDR__X_ORIGINAL_MESSAGE_ID	38
+#define HDR__CANCEL_LOCK	39
+#define HDR__CONTENT_TRANSFER_ENCODING	40
+#define HDR__FACE		41
+#define HDR__INJECTION_INFO	42
+#define HDR__LIST_ID		43
+#define HDR__MIME_VERSION	44
+#define HDR__ORIGINATOR		45
+#define HDR__X_AUTH		46
+#define HDR__X_COMPLAINTS_TO	47
+#define HDR__X_FACE		48
+#define HDR__X_HTTP_USERAGENT	49
+#define HDR__X_HTTP_VIA		50
+#define HDR__X_MODBOT		51
+#define HDR__X_MODTRACE		52
+#define HDR__X_NO_ARCHIVE	53
+#define HDR__X_ORIGINAL_TRACE	54
+#define HDR__X_ORIGINATING_IP	55
+#define HDR__X_PGP_KEY		56
+#define HDR__X_PGP_SIG		57
+#define HDR__X_POSTER_TRACE	58
+#define HDR__X_POSTFILTER	59
+#define HDR__X_PROXY_USER	60
+#define HDR__X_SUBMISSIONS_TO	61
+#define HDR__X_USENET_PROVIDER	62
+#define HDR__IN_REPLY_TO	63
+#define HDR__INJECTION_DATE	64
+#define HDR__NNTP_POSTING_DATE	65
 
-#define MAX_ARTHEADER		39
+#define MAX_ARTHEADER           66
 
 /*
 **  Miscellaneous data we want to keep on an article.  All the fields
@@ -240,6 +267,8 @@ typedef struct _ARTDATA {
   bool            AddAlias;             /* Whether Pathalias should be added
                                            to this article */
   bool            Hassamepath;          /* Whether this article matches Path */
+  bool            Hassamecluster;       /* Whether this article matches 
+                                           Pathcluster */
 } ARTDATA;
 
 /*
@@ -406,6 +435,22 @@ typedef enum _FEEDTYPE {
 } FEEDTYPE;
 
 
+/*
+**  Diablo-style hashed feeds or hashfeeds.
+*/
+#define HASHFEED_QH	1
+#define HASHFEED_MD5	2
+
+typedef struct _HASHFEEDLIST {
+  int			type;
+  unsigned int		begin;
+  unsigned int		end;
+  unsigned int		mod;
+  unsigned int		offset;
+  struct _HASHFEEDLIST	*next;
+} HASHFEEDLIST;
+
+
 /*
 **  A site may reject something in its subscription list if it has
 **  too many hops, or a bad distribution.
@@ -458,6 +503,7 @@ typedef struct _SITE {
   struct buffer	  Buffer;
   bool		  Buffered;
   char	      **  Originator;
+  HASHFEEDLIST *  HashFeedList;
   int		  Next;
   int		  Prev;
 } SITE;
@@ -567,6 +613,7 @@ extern bool		StreamingOff;
 extern bool		Tracing;
 EXTERN struct buffer	Path;
 EXTERN struct buffer	Pathalias;
+EXTERN struct buffer    Pathcluster;
 EXTERN char	     *  ModeReason;	/* NNTP reject message   */
 EXTERN char	     *  NNRPReason;	/* NNRP reject message   */
 EXTERN char	     *  Reservation;	/* Reserved lock message */
