@@ -1,4 +1,4 @@
-/*  $Id: status.c 6372 2003-05-31 19:48:28Z rra $
+/*  $Id: status.c 7547 2006-08-26 06:18:14Z eagle $
 **
 **  Periodic status reporting.
 */
@@ -122,8 +122,10 @@ STATUSsummary(void)
 #else
   path = concatpath(innconf->pathlog, STATUS_FILE);
 #endif
-  if ((F = Fopen(path, "w", TEMPORARYOPEN)) == NULL)
+  if ((F = Fopen(path, "w", TEMPORARYOPEN)) == NULL) {
+    syslog(L_ERROR, "%s cannot open %s: %m", LogName, path);
     return;
+  }
 
 #if defined(HTML_STATUS)
   /* HTML Header */
