@@ -1,4 +1,4 @@
-/*  $Id: endpoint.c,v 1.31.2.1 2004/01/25 20:07:10 rra Exp $
+/*  $Id: endpoint.c 7738 2008-04-06 09:33:33Z iulius $
 **
 **  The implementation of the innfeed EndPoint object class.
 **
@@ -1644,7 +1644,7 @@ void setSigHandler (int signum, void (*ptr)(int))
   if (sigHandlers == NULL)
     {
       sigHandlers = xmalloc (sizeof(sigfn) * NSIG) ;
-      sigFlags = xmalloc (sizeof(int) * NSIG) ;
+      sigFlags = xmalloc (sizeof(sig_atomic_t) * NSIG) ;
       for (i = 0 ; i < NSIG ; i++)
         {
           sigHandlers [i] = NULL ;
