@@ -701,7 +701,7 @@ static void closehashtable(hash_table *tab) {
 	free(tab->core);
     if (tab->incore == INCORE_MMAP) {
 #if defined(HAVE_MMAP)
-	if (munmap(tab->core, (int)conf.tsize * tab->reclen) == -1) {
+	if (munmap(tab->core, conf.tsize * tab->reclen) == -1) {
 	    syswarn("closehashtable: munmap failed");
 	}
 #else
@@ -1246,8 +1246,8 @@ static bool
 getcore(hash_table *tab)
 {
     char *it;
-    int nread;
-    int i;
+    ssize_t nread;
+    size_t i;
     struct stat st;
     size_t length = conf.tsize * tab->reclen;
 
@@ -1303,14 +1303,16 @@ getcore(hash_table *tab)
 static bool
 putcore(hash_table *tab)
 {
-    int size;
+    size_t size;
+    ssize_t result;
     
     if (tab->incore == INCORE_MEM) {
 	if(options.writethrough)
 	    return true;
 	nonblocking(tab->fd, false);
 	size = tab->reclen * conf.tsize;
-	if (pwrite(tab->fd, tab->core, size, 0) != size) {
+	result = xpwrite(tab->fd, tab->core, size, 0);
+	if (result < 0 || (size_t) result != size) {
 	    nonblocking(tab->fd, options.nonblock);
 	    return false;
 	}
@@ -1318,7 +1320,7 @@ putcore(hash_table *tab)
     }
 #ifdef HAVE_MMAP
     if(tab->incore == INCORE_MMAP) {
-	msync(tab->core, (int)conf.tsize * tab->reclen, MS_ASYNC);
+	msync(tab->core, conf.tsize * tab->reclen, MS_ASYNC);
     }
 #endif
     return true;
@@ -1540,7 +1542,7 @@ set(searcher *sp, hash_table *tab, void *value)
 	debug("set: incore");
 	if (tab->incore == INCORE_MMAP) {
 	    if (innconf->nfswriter) {
-		mapcntl(where, tab->reclen, MS_ASYNC);
+		inn_mapcntl(where, tab->reclen, MS_ASYNC);
 	    }
 	    return true;
 	}
