@@ -1,15 +1,15 @@
-/*  $Revision: 1.4 $
+/*  $Id: defdist.c 6135 2003-01-19 01:15:40Z rra $
 **
 */
-#include <stdio.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <errno.h>
-#include "configdata.h"
-#include "libinn.h"
+
+#include "config.h"
 #include "clibrary.h"
+#include <ctype.h>
+#include <errno.h>
+
+#include "inn/innconf.h"
+#include "libinn.h"
 #include "paths.h"
-#include "macros.h"
 
 
 typedef struct _DDENTRY {
@@ -25,13 +25,8 @@ struct _DDHANDLE {
 };
 typedef struct _DDHANDLE	DDHANDLE;
 
-
-extern FILE	*CA_listopen();
-
 struct _DDHANDLE *
-DDstart(FromServer, ToServer)
-    FILE	*FromServer;
-    FILE	*ToServer;
+DDstart(FILE *FromServer, FILE *ToServer)
 {
     DDHANDLE	*h;
     DDENTRY	*ep;
@@ -39,16 +34,26 @@ DDstart(FromServer, ToServer)
     char	buff[BUFSIZ];
     char	*p;
     char	*q;
+    char        *path;
     int		i;
-    char	name[sizeof _PATH_TEMPACTIVE];
+    int         fd;
+    char	*name = NULL;
 
     /* Open the file. */
-    if ((F = fopen(_PATH_DISTPATS, "r")) != NULL)
-	name[0] = '\0';
-    else {
+    path = concatpath(innconf->pathetc, _PATH_DISTPATS);
+    F = fopen(path, "r");
+    free(path);
+    if (F == NULL) {
 	/* Not available locally; try remotely. */
-	(void)strcpy(name, _PATH_TEMPACTIVE);
-	(void)mktemp(name);
+	if (FromServer == NULL || ToServer == NULL)
+	    /* We're probably nnrpd running on the server and the
+	     * file isn't installed.  Oh well. */
+	    return NULL;
+        name = concatpath(innconf->pathtmp, _PATH_TEMPACTIVE);
+        fd = mkstemp(name);
+        if (fd < 0)
+            return NULL;
+        close(fd);
 	if ((F = CA_listopen(name, FromServer, ToServer,
 		    "distrib.pats")) == NULL)
 	    return NULL;
@@ -59,27 +64,29 @@ DDstart(FromServer, ToServer)
 	continue;
 
     /* Allocate space for the handle. */
-    if ((h = NEW(DDHANDLE, 1)) == NULL) {
+    if ((h = xmalloc(sizeof(DDHANDLE))) == NULL) {
 	i = errno;
-	(void)fclose(F);
-	if (name[0])
-	    (void)unlink(name);
+	fclose(F);
+	if (name != NULL)
+	    unlink(name);
 	errno = i;
 	return NULL;
     }
     h->Count = 0;
     h->Current = NULL;
-    if ((h->Entries = NEW(DDENTRY, i)) == NULL) {
+    if (i == 0) {
+        return NULL ;
+    } else if ((h->Entries = xmalloc(sizeof(DDENTRY) * i)) == NULL) {
 	i = errno;
-	DISPOSE(h);
-	(void)fclose(F);
-	if (name[0])
-	    (void)unlink(name);
+	free(h);
+	fclose(F);
+	if (name != NULL)
+	    unlink(name);
 	errno = i;
 	return NULL;
     }
 
-    (void)fseek(F, (OFFSET_T)0, SEEK_SET);
+    fseeko(F, 0, SEEK_SET);
     for (ep = h->Entries; fgets(buff, sizeof buff, F) != NULL; ) {
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
@@ -90,7 +97,7 @@ DDstart(FromServer, ToServer)
 	    continue;
 	*p++ = '\0';
 	ep->Weight = atoi(buff);
-	ep->Pattern = COPY(p);
+	ep->Pattern = xstrdup(p);
 	q = strchr(ep->Pattern, ':');
 	*q++ = '\0';
 	ep->Value = q;
@@ -98,17 +105,15 @@ DDstart(FromServer, ToServer)
     }
     h->Count = ep - h->Entries;
 
-    (void)fclose(F);
-    if (name[0])
-	(void)unlink(name);
+    fclose(F);
+    if (name != NULL)
+	unlink(name);
     return h;
 }
 
 
 void
-DDcheck(h, group)
-    DDHANDLE	*h;
-    char	*group;
+DDcheck(DDHANDLE *h, char *group)
 {
     DDENTRY	*ep;
     int		i;
@@ -119,14 +124,15 @@ DDcheck(h, group)
 
     w = h->Current ? h->Current->Weight : -1;
     for (ep = h->Entries, i = h->Count; --i >= 0; ep++)
-	if (ep->Weight > w && wildmat(group, ep->Pattern))
+	if (ep->Weight > w && uwildmat(group, ep->Pattern)) {
 	    h->Current = ep;
+	    w = ep->Weight;
+	}
 }
 
 
 char *
-DDend(h)
-    DDHANDLE	*h;
+DDend(DDHANDLE *h)
 {
     static char	NIL[] = "";
     char	*p;
@@ -135,31 +141,45 @@ DDend(h)
 
     if (h == NULL) {
 	p = NIL;
-	return COPY(p);
+	return xstrdup(p);
     }
 
     if (h->Current == NULL)
 	p = NIL;
     else
 	p = h->Current->Value;
+    p = xstrdup(p);
 
     for (ep = h->Entries, i = h->Count; --i >= 0; ep++)
-	DISPOSE(ep->Pattern);
-    DISPOSE(h->Entries);
-    DISPOSE(h);
-    return COPY(p);
+	free(ep->Pattern);
+    free(h->Entries);
+    free(h);
+    return p;
 }
 
 #if	defined(TEST)
 int
-main(ac, av)
-    int			ac;
-    char		*av[];
+main(int ac, char *av[])
 {
     struct _DDHANDLE	*h;
     char		*p;
+    FILE		*FromServer;
+    FILE		*ToServer;
+    char		buff[SMBUF];
 
-    if ((h = DDstart()) == NULL)
+    if (NNTPremoteopen(NNTP_PORT, &FromServer, &ToServer, buff) < 0) {
+	if ((p = strchr(buff, '\n')) != NULL)
+	    *p = '\0';
+	if ((p = strchr(buff, '\r')) != NULL)
+	    *p = '\0';
+	if (buff[0])
+	    fprintf(stderr, "%s\n", buff);
+	else
+	    perror("Can't connect");
+	exit(1);
+    }
+
+    if ((h = DDstart(FromServer, ToServer)) == NULL)
 	perror("Init failed, proceeding anyway");
     while ((p = *++av) != NULL)
 	DDcheck(h, p);
