@@ -2,16 +2,14 @@
    is implemented using MD5, but should probably have a mechanism for
    choosing the hash algorithm and tagging the hash with the algorithm
    used */
-#include <stdio.h>
-#include <sys/types.h>
-#include <ctype.h>
-#include "configdata.h"
+#include "config.h"
 #include "clibrary.h"
-#include "libinn.h"
-#include "macros.h"
-#include "md5.h"
+#include <ctype.h>
 
-STATIC HASH empty= { { 0, 0, 0, 0, 0, 0, 0, 0,
+#include "inn/md5.h"
+#include "libinn.h"
+
+static HASH empty= { { 0, 0, 0, 0, 0, 0, 0, 0,
 		       0, 0, 0, 0, 0, 0, 0, 0 }};
 
 /* cipoint - where in this message-ID does it become case-insensitive?
@@ -27,13 +25,15 @@ STATIC HASH empty= { { 0, 0, 0, 0, 0, 0, 0, 0,
  *
  * Returns: pointer into s, or NULL for "nowhere"
  */
-STATIC char *cipoint(char *s, size_t size) {
+static const char *
+cipoint(const char *s, size_t size)
+{
     char *p;
-    static char post[] = "postmaster";
+    static const char post[] = "postmaster";
     static int plen = sizeof(post) - 1;
 
-    if ((p = memchr(s, '@', size))== NULL)			/* no local/domain split */
-	return NULL;		/* assume all local */
+    if ((p = memchr(s, '@', size))== NULL)	/* no local/domain split */
+	return NULL;				/* assume all local */
     if ((p - (s + 1) == plen) && !strncasecmp(post, s+1, plen)) {
 	/* crazy -- "postmaster" is case-insensitive */
 	return s;
@@ -41,42 +41,46 @@ STATIC char *cipoint(char *s, size_t size) {
     return p;
 }
 
-HASH Hash(const void *value, const size_t len) {
-    MD5_CTX context;
+HASH
+Hash(const void *value, const size_t len)
+{
+    struct md5_context context;
     HASH hash;
 
-    MD5Init(&context);
-    MD5Update(&context, value, len);
-    MD5COUNT(&context, len);
-    MD5Final(&context);
+    md5_init(&context);
+    md5_update(&context, value, len);
+    md5_final(&context);
     memcpy(&hash,
 	   &context.digest,
 	   (sizeof(hash) < sizeof(context.digest)) ? sizeof(hash) : sizeof(context.digest));
     return hash;
 }
 
-HASH HashMessageID(const char *MessageID) {
-    static char         *new = NULL;
-    static int          newlen = 0;
-    char                *cip;
-    char                *p;
+HASH
+HashMessageID(const char *MessageID)
+{
+    char                *new = NULL;
+    const char          *cip, *p = NULL;
+    char                *q;
     int                 len;
     HASH                hash;
 
     len = strlen(MessageID);
-    if (newlen <= len) {
-	if (new)
-	    RENEW(new, char, len + 1);
-	else
-	    new = NEW(char, len + 1);
-	newlen = len + 1;
+    cip = cipoint(MessageID, len);
+    if (cip != NULL) {
+        for (p = cip + 1; *p != '\0'; p++) {
+            if (!CTYPE(islower, *p)) {
+                new = xstrdup(MessageID);
+                break;
+            }
+        }
     }
-    strcpy(new, MessageID);
-    if ((cip = cipoint(new, len))) {
-	for (p = cip + 1; *p; p++)
-	    *p = tolower(*p);
-    }
-    hash = Hash(new, len);
+    if (new != NULL)
+        for (q = new + (p - MessageID); *q != '\0'; q++)
+            *q = tolower(*q);
+    hash = Hash(new ? new : MessageID, len);
+    if (new != NULL)
+	free(new);
     return hash;
 }
 
@@ -84,7 +88,9 @@ HASH HashMessageID(const char *MessageID) {
 **  Check if the hash is all zeros, and subseqently empty, see HashClear
 **  for more info on this.
 */
-BOOL HashEmpty(const HASH h) {
+bool
+HashEmpty(const HASH h)
+{
     return (memcmp(&empty, &h, sizeof(HASH)) == 0);
 }
 
@@ -93,7 +99,9 @@ BOOL HashEmpty(const HASH h) {
 **  introduces the possibility of colliding w/a value that actually hashes
 **  to all zeros, but that's fairly unlikely.
 */
-void HashClear(HASH *hash) {
+void
+HashClear(HASH *hash)
+{
     memset(hash, '\0', sizeof(HASH));
 }
 
@@ -101,13 +109,15 @@ void HashClear(HASH *hash) {
 **  Convert the binary form of the hash to a form that we can use in error
 **  messages and logs.
 */
-char *HashToText(const HASH hash) {
-    STATIC char         hex[] = "0123456789ABCDEF";
-    char                *p;
-    STATIC char         hashstr[(sizeof(HASH) * 2) + 1];
-    int                 i;
+char *
+HashToText(const HASH hash)
+{
+    static const char	hex[] = "0123456789ABCDEF";
+    const char		*p;
+    unsigned int	i;
+    static char		hashstr[(sizeof(HASH)*2) + 1];
 
-    for (p = (char *)&hash, i = 0; i < sizeof(HASH); i++, p++) {
+    for (p = hash.hash, i = 0; i < sizeof(HASH); i++, p++) {
 	hashstr[i * 2] = hex[(*p & 0xF0) >> 4];
 	hashstr[(i * 2) + 1] = hex[*p & 0x0F];
     }
@@ -118,14 +128,18 @@ char *HashToText(const HASH hash) {
 /*
 ** Converts a hex digit and converts it to a int
 */
-STATIC int hextodec(const int c) {
+static
+int hextodec(const int c)
+{
     return isdigit(c) ? (c - '0') : ((c - 'A') + 10);
 }
 
 /*
 **  Convert the ASCII representation of the hash back to the canonical form
 */
-HASH TextToHash(const char *text) {
+HASH
+TextToHash(const char *text)
+{
     char                *q;
     int                 i;
     HASH                hash;
