@@ -1,26 +1,28 @@
-/*  $Revision: 1.5 $
+/*  $Id: localopen.c 6155 2003-01-19 19:58:25Z rra $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
-#include <sys/socket.h>
 #include <errno.h>
-#include "paths.h"
-#include "nntp.h"
-#include "macros.h"
-#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
-#include <sys/un.h>
-#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
+#include <sys/socket.h>
+
+#include "inn/innconf.h"
 #include "libinn.h"
+#include "nntp.h"
+#include "paths.h"
+
+#if HAVE_UNIX_DOMAIN_SOCKETS
+# include <sys/un.h>
+#endif
 
 
 /*
 **  Open a connection to the local InterNetNews NNTP server and optionally
 **  create stdio FILE's for talking to it.  Return -1 on error.
 */
-int NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
+int
+NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
 {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     int			i;
@@ -39,14 +41,14 @@ int NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
 	return -1;
 
     /* Connect to the server. */
-    (void)memset((POINTER)&server, 0, sizeof server);
+    memset(&server, 0, sizeof server);
     server.sun_family = AF_UNIX;
-    (void)strcpy(server.sun_path, innconf->pathrun);
-    (void)strcat(server.sun_path, "/");
-    (void)strcat(server.sun_path, _PATH_NNTPCONNECT);
-    if (connect(i, (struct sockaddr *)&server, AF_UNIX_SOCKSIZE(server)) < 0) {
+    strlcpy(server.sun_path, innconf->pathrun, sizeof(server.sun_path));
+    strlcat(server.sun_path, "/", sizeof(server.sun_path));
+    strlcat(server.sun_path, _PATH_NNTPCONNECT, sizeof(server.sun_path));
+    if (connect(i, (struct sockaddr *)&server, SUN_LEN(&server)) < 0) {
 	oerrno = errno;
-	(void)close(i);
+	close(i);
 	errno = oerrno;
 	return -1;
     }
@@ -54,19 +56,19 @@ int NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
     /* Connected -- now make sure we can post. */
     if ((F = fdopen(i, "r")) == NULL) {
 	oerrno = errno;
-	(void)close(i);
+	close(i);
 	errno = oerrno;
 	return -1;
     }
     if (fgets(buff, sizeof mybuff, F) == NULL) {
 	oerrno = errno;
-	(void)fclose(F);
+	fclose(F);
 	errno = oerrno;
 	return -1;
     }
     j = atoi(buff);
     if (j != NNTP_POSTOK_VAL && j != NNTP_NOPOSTOK_VAL) {
-	(void)fclose(F);
+	fclose(F);
 	/* This seems like a reasonable error code to use... */
 	errno = EPERM;
 	return -1;
@@ -75,12 +77,13 @@ int NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
     *FromServerp = F;
     if ((*ToServerp = fdopen(dup(i), "w")) == NULL) {
 	oerrno = errno;
-	(void)fclose(F);
+	fclose(F);
 	errno = oerrno;
 	return -1;
     }
     return 0;
 #else
-    return NNTPconnect(LOOPBACK_HOST, innconf->port, FromServerp, ToServerp, errbuff);
+    return NNTPconnect("127.0.0.1", innconf->port, FromServerp, ToServerp,
+                       errbuff);
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 }
