@@ -1,4 +1,4 @@
-/*  $Id: perl.c,v 1.41 2003/02/10 03:21:27 rra Exp $
+/*  $Id: perl.c 7815 2008-05-05 08:43:58Z iulius $
 **
 **  Embedded Perl support for INN.
 **
@@ -63,6 +63,7 @@ char *HandleHeaders(char *article)
    HE            *scan;
    SV            *modswitch;
    int            OtherSize;
+   char *argv[] = { NULL };
 
    if(!PerlLoaded) {
        loadPerl();
@@ -112,7 +113,7 @@ char *HandleHeaders(char *article)
    sv_setpv(body, article);
 
    /* Call the filtering function */
-   rc = perl_call_argv("filter_post", G_EVAL|G_SCALAR, NULL);
+   rc = perl_call_argv("filter_post", G_EVAL|G_SCALAR, argv);
 
    SPAGAIN;
 
