@@ -1,4 +1,4 @@
-/*  $Id: trash.h,v 1.5.2.2 2001/02/03 07:29:26 rra Exp $
+/*  $Id: trash.h 4267 2001-01-04 06:02:02Z rra $
 **
 **  trashing articles method header
 */
@@ -6,17 +6,17 @@
 #ifndef __TRASH_H__
 #define __TRASH_H__
 
-#include <configdata.h>
-#include <interface.h>
+#include "config.h"
+#include "interface.h"
 
-BOOL trash_init(SMATTRIBUTE *attr);
+bool trash_init(SMATTRIBUTE *attr);
 TOKEN trash_store(const ARTHANDLE article, const STORAGECLASS class);
 ARTHANDLE *trash_retrieve(const TOKEN token, const RETRTYPE amount);
 ARTHANDLE *trash_next(const ARTHANDLE *article, const RETRTYPE amount);
 void trash_freearticle(ARTHANDLE *article);
-BOOL trash_cancel(TOKEN token);
-BOOL trash_ctl(PROBETYPE type, TOKEN *token, void *value);
-BOOL trash_flushcacheddata(FLUSHTYPE type);
+bool trash_cancel(TOKEN token);
+bool trash_ctl(PROBETYPE type, TOKEN *token, void *value);
+bool trash_flushcacheddata(FLUSHTYPE type);
 void trash_printfiles(FILE *file, TOKEN token, char **xref, int ngroups);
 void trash_shutdown(void);
 
