@@ -1,4 +1,4 @@
-/* $Id: snprintf-t.c 6128 2003-01-18 22:26:49Z rra $ */
+/* $Id: snprintf-t.c 7510 2006-04-02 18:31:51Z eagle $ */
 /* snprintf test suite. */
 
 #include "config.h"
@@ -125,7 +125,7 @@ main(void)
     test_format(21, true, "abcdefghij4444", 14, "%.10s%n%d", string, &count,
                 4444);
     ok(22, count == 10);
-    test_format(23, true, "abcdefghijklmnopqrstuvwxyz01234", 36, "%ln%s%n",
+    test_format(23, true, "abcdefghijklmnopqrstuvwxyz01234", 36, "%n%s%ln",
                 &count, string, &lcount);
     ok(24, count == 0);
     ok(25, lcount == 31);
